#######################################################################################
#
# Computer codes accompanying Critique of the methods employed to project global sea
# level rise from global temperature. PNAS 107, E116-E117, doi: 10.1073/pnas.0914942107
#
# Fernando Gonzlez Taboada & Ricardo Anadn
# (fgtaboda@gmail.com & ranadon@uniovi.es)
#
# Here we present some calculations to support our criticism to the paper by
# Vermeer and Rahmstorf (2009). The codes should be run using in R (R Development Core
# Team (2010)).
# The accompanying data files "sealevel.txt" and "ipccAR4_scenarios.txt" should be
# in the same folder as the codes, otherwise, use the command 'setwd' to change
# the working directory in R (see ?setwd) to load the data.
#
# We have adapted Vermeer and Rahmstorf (2009) data to be easily imported and
# managed within R. All the transformations applied to the data have
# been taken from Rahmstorf and Vermeer codes, available at
# http://www.pnas.org/content/early/2009/12/04/0907765106/suppl/DCSupplemental.
# We would like to acknowledge Vermeer and Rahmstorf transparency in sharing
# their data and codes.
#
# The code is free to use for any one interested, although the user should consider
# that we can only offer limited support in case of probelms running the code (contact
# us preferably by email), and that the computations performed by these codes are not
# an example of best modelling practices.
#
# The file sealevel.txt contains seven variables:
#
#     t is time in years
#     H is global mean sea level, expressed in cm. Data comes from Church
#   and White (2006) reconstruction (see also Church et al 2004); and is
#   available at the Permanent Service for Mean Sea Level (PMSL) website,
#   hosted at the Proudman Oceanographic Laboratory; see
#   http://www.pol.ac.uk/psmsl/author_archive/church_white/.
#   Chao et al (2008) reservoir correction has been applied, and following
#   Vermeer and Rahmstorf (2009), the smoothed (see variable type) level at
#   1990 has been substracted
#     dH is the rate of sea level change, in cm by year, estimated by finite
#   differences using H values
#     T is global mean temperature, expressed as anomalies computed relative
#   to 1951-1980 in Celsius degrees. The dataset is produced and updated at
#   NASA Goddard Institute for Space Studies (GISTEMP, see for instance
#   Hansen et al 2001), and it is available at http://data.giss.nasa.gov/gistemp/.
#   The website provides also detailed information and references.
#     dT is the rate of temperature change, in Celsius degrees by year, esti-
#   mated by finite differences using T values.
#     cT is the cumulated sum of temperature anomalies, in Celsius degrees,
#   calculated using T values from 1880, the first year considered in the analysis
#     type is a nomimal variable indicating for each record whether it correspond
#   to 'Observed', 'Smoothed' or 'Binned' data. 'Observed' data correspond nearly
#   to raw data, while 'Smoothed' data was derived using Singular Spectrum Analy-
#   sis, using the procedures and code described in Moore et al (2005) (see also,
#   http://www.glaciology.net/software/ssatrend-m). Note that the mean, smoothed
#   sea level for 1990 was substracted from both the 'Observed' and 'Smoothed'
#   series. 'Binned' data refers to averages taken using a 15 year window over
#   'Smoothed' data for each variable.
#
# The file ipccAR4_scenarios.txt contains the following variables
#
#     t is time in years (1990-2100)
#     model is a categorical variable coding each of the 19 cliamte models considered
#     scenario is a categorical variable coding the IPCC AR4 scenario to which pT
#   projections correspond. Six scenarios were considered: 'A1B', 'A1FI', 'A1T', 'A2',
#   'B1', and 'B2'.
#     ccycle is a categorical variable coding the type of carbon cycle feedbacks assumed,
#   with three levels: 'low', 'default' and 'high'
#     dT is the rate of temperature change, in Celsius degrees by year, estimated
#   by finite differences using T values.
#     T are temperature increments projected by each model adjusted to GISTEMP series
#   of temperature anomalies (e.g. same mean between 1880 and 1920)
#     cT is the cumulated sum of temperature anomalies, in Celsius degrees,
#   calculated using T values from 1880, the first year considered in the analysis
#   (e.g. -9.9145512, the cT value in 1990 calculated from smoothed data  was added
#   to the IPCC series of predicted increments)
#
# ## Last test: 2010-06-10 ##
#
# References
#
# Auguie B (2010) gridExtra: functions in Grid graphics. R, package version 0.6.
#     http://CRAN.R-project.org/package=gridExtra
# Chao B F, Wu Y H, Li Y S (2008) Impact of artificial reservoir water impoundment on
#     global sea level. Science, 320, 212-214.
# Church J A, White N J, Coleman R, Lambeck K, Mitrovica J X (2004) Estimates of the
#     regional distribution of sea-level rise over the 1950 to 2000 period. J Clim 17:
#     2609-2625.
# Church J A, White N J (2006) A 20th century acceleration in global sea-level rise.
#     Geophys Res Lett 33: L01602, doi:10.1029/2005GL024826.
# Eddelbuettel D (contrib. Lucas A, Tuszynski J, Bengtsson H, Urbanek S) (2009).
#     digest: Create cryptographic hash digests of R objects. R package version
#     0.4.2. http://CRAN.R-project.org/package=digest
# Hansen J E, Ruedy R, Sato M, Imhoff M, Lawrence W, Easterling D, Peterson T,
#     Karl T (2001) A closer look at United States and global surface temperature
#     change. J Geophys Res 106: 23947-23963, doi:10.1029/2001JD000354.
# Intergovernmental Panel on Climate Change (2007) The Fourth Assessment Report
#     of the Intergovernmental Panel on Climate Change, eds Solomon S, et al.
#     (Cambridge Univ Press, Cambridge UK).
# Kates L, Petzoldt T (2007) proto: Prototype object-based programming. R package
#     version 0.3-8. http://code.google.com/p/r-proto/
# Moore J C, Grinsted A, Jevrejeva S (2005) New Tools for Analyzing Time Series
#     Relationships and Trends. EOS, 86(24).
# R Development Core Team (2010). R: A language and environment for statistical
#     computing. R Foundation for Statistical Computing, Vienna, Austria. URL
#     http://www.R-project.org
# Rahmstorf S (2007) A semi-empirical approach to projecting future sea-level rise.
#     Science 315: 368-370.
# Vermeer M, Rahmstorf S (2009) Global sea level linked to global temperature.
#     Proc Natl Acad Sci USA, doi:10.1073/pnas.0907765106.
# Wickham H (2007) Reshaping data with the reshape package. J Stat Soft 21(12).
# Wickham H (2010) plyr: Tools for splitting, applying and combining data. R package
#     version 1.0.3. http://CRAN.R-project.org/package=plyr
# Wickham H (2009) ggplot2: elegant graphics for data analysis. Springer, New York, USA.
#
#######################################################################################


# Load packages necessary to plot the data and to conduct analysis
require(digest)
require(ggplot2)
require(gridExtra)
require(reshape)
require(plyr)
require(proto)

# Otherwise
# install.packages(c("ggplot2"))
#install.packages("gridExtra", repos="http://R-Forge.R-project.org")


# Packages loaded and versions (the comment indicates package versions used to test
# the codes appart than base packages for R version 2.11.1 (2010-05-31))
sessionInfo()
# digest 0.4.2
# ggplot2 0.8.8
# gridExtra 0.6
# reshape 0.8.3
# plyr 1.0.3
# proto 0.3.8


# change working directory to the folder containing the data if necessary
#setwd("C:\\myfolderwithdata")
setwd("C:\\Users\\fgtaboada\\Desktop\\pnes\\Ricardo web")
# Load sea level and temperature data; please note the comments about the data above
data <- read.table("sealevel.txt")
str(data)

##############################################################################################
## Fig. 1 (Letter) Relationships between sea level (H[t]) and the cumulated temperature
# anomaly (sum(T[i], 0, t)), and between H[t] and the temperature anomaly (T[t]). The panels
# present the same relationship between raw (small, light grey points) and smoothed and binned
# (large, dark grey points) data. The lines and ribbons correspond to linear regression and
# confidence intervals estimated for smoothed and binned data, and they have been included to
# illustrate the expected sign of a hypothetical linear relationship between the variables.
# Note that, in both refs. 1 and 2, the effect of this variables on sea level was teh opposite
# than that shown here.
x11(7.3, 3.5)
#png("fig1.png", width = 11, height = 7.5, units = "in", res = 600)
#pdf("newfig1.pdf", width = 7.3, height = 3.5)
a1 <- ggplot(aes(x = cT, y = H), data = subset(data, type == "Observed")) +
    geom_point(data = subset(data, type == "Observed"),
           shape = I(21), colour = I("grey40"), fill = I("grey75"), size = I(2.25)) +
    geom_point(data = subset(data, type == "Binned"),
           shape = I(21), colour = I("grey20"), fill = I("grey50"), size = I(4)) +
    geom_smooth(data = subset(data, type == "Binned"),
           method = "lm", colour = I("grey20"), fill = I("grey50")) +
    scale_x_continuous(name = expression(Cumulated~temperature~anomaly~group("(", degree*C, ")")), limits = c(-13,0)) +
    scale_y_continuous(name = expression(Sea~level~group("(", cm,")")), limits = c(-27.5,7.5)) +
    theme_bw()

b1 <- ggplot(aes(x = T, y = H), data = subset(data, type == "Observed")) +
    geom_point(data = subset(data, type == "Observed"),
           shape = I(21), colour = I("grey40"), fill = I("grey75"), size = I(2.25)) +
    geom_point(data = subset(data, type == "Binned"),
           shape = I(21), colour = I("grey20"), fill = I("grey50"), size = I(4)) +
    geom_smooth(data = subset(data, type == "Binned"),
           method = "lm", colour = I("grey20"), fill = I("grey50")) +
    scale_x_continuous(name = expression(Temperature~anomaly~group("(",degree*C,")")), limits = c(-0.4,0.5)) +
    scale_y_continuous(name = expression(Sea~level~group("(", cm,")")), limits = c(-27.5,7.5)) +
    theme_bw()
arrange(a1, b1, nrow = 1)
#dev.off()


##############################################################################################
## Fig. S0 Relationship between sea level (H[t]) and the cumulated temperature
# anomaly (sum(T[i], 0, t)), and between H[t] and the temperature anomaly (T[t]). From left
# to right, panels present the same relationship between raw, smoothed, and smoothed and binned
# data. Brown lines and ribbons represent a linear regression with confidence intervals, and have
# been included to illustrate the expected sign of a hypothetical linear  relationship between
# the variables. Note that in both refs. (1) and (2), the effect of this variables on sea level
# was of opposed sign than that shown here.
x11(11, 7)
#png("fig1.png", width = 11, height = 7.5, units = "in", res = 600)
a1 <- qplot(x = cT, y = H, data = subset(data, type == "Observed"),
           pch = 20, colour = I("cadetblue"), size = I(3)) +
    geom_smooth(method = "lm", fill = "antiquewhite3", colour = "antiquewhite4") +
    geom_text(x = -12, y = 5, label = "Observed", vjust = 0, hjust = 0, size = 6.5) +
    scale_x_continuous(name = expression(Cumulated~temperature~anomaly~group("(", degree*C, ")")), limits = c(-13,0)) +
    scale_y_continuous(name = expression(Sea~level~group("(", cm,")")), limits = c(-27.5,7.5)) +
    theme_bw()
a2 <- qplot(x = cT, y = H, data = subset(data, type == "Smoothed"),
           pch = 20, colour = I("cadetblue"), size = I(3)) +
    geom_smooth(method = "lm", fill = "antiquewhite3", colour = "antiquewhite4") +
    geom_text(x = -12, y = 5, label = "Smoothed", vjust = 0, hjust = 0, size = 6.5) +
    scale_x_continuous(name = expression(Cumulated~temperature~anomaly~group("(", degree*C, ")")), limits = c(-13,0)) +
    scale_y_continuous(name = expression(Sea~level~group("(", cm,")")), limits = c(-27.5,7.5)) +
    theme_bw()
a3 <- qplot(x = cT, y = H, data = subset(data, type == "Binned"),
           pch = 20, colour = I("cadetblue"), size = I(3)) +
    geom_smooth(method = "lm", fill = "antiquewhite3", colour = "antiquewhite4") +
    geom_text(x = -12, y = 5, label = "Binned", vjust = 0, hjust = 0, size = 6.5) +
    scale_x_continuous(name = expression(Cumulated~temperature~anomaly~group("(", degree*C, ")")), limits = c(-13,0)) +
    scale_y_continuous(name = expression(Sea~level~group("(", cm,")")), limits = c(-27.5,7.5)) +
    theme_bw()
b1 <- qplot(x = T, y = H, data = subset(data, type == "Observed"),
           pch = 20, colour = I("cadetblue"), size = I(3)) +
    geom_smooth(method = "lm", fill = "antiquewhite3", colour = "antiquewhite4") +
    geom_text(x = -0.34, y = 5, label = "Observed", vjust = 0, hjust = 0, size = 6.5) +
    scale_x_continuous(name = expression(Temperature~anomaly~group("(",degree*C,")")), limits = c(-0.4,0.5)) +
    scale_y_continuous(name = expression(Sea~level~group("(", cm,")")), limits = c(-27.5,7.5)) +
    theme_bw()
b2 <- qplot(x = T, y = H, data = subset(data, type == "Smoothed"),
           pch = 20, colour = I("cadetblue"), size = I(3)) +
    geom_smooth(method = "lm", fill = "antiquewhite3", colour = "antiquewhite4") +
    geom_text(x = -0.34, y = 5, label = "Smoothed", vjust = 0, hjust = 0, size = 6.5) +
    scale_x_continuous(name = expression(Temperature~anomaly~group("(",degree*C,")")), limits = c(-0.4,0.5)) +
    scale_y_continuous(name = expression(Sea~level~group("(", cm,")")), limits = c(-27.5,7.5)) +
    theme_bw()
b3 <- qplot(x = T, y = H, data = subset(data, type == "Binned"),
           pch = 20, colour = I("cadetblue"), size = I(3)) +
    geom_smooth(method = "lm", fill = "antiquewhite3", colour = "antiquewhite4") +
    geom_text(x = -0.34, y = 5, label = "Binned", vjust = 0, hjust = 0, size = 6.5) +
    scale_x_continuous(name = expression(Temperature~anomaly~group("(",degree*C,")")), limits = c(-0.4,0.5)) +
    scale_y_continuous(name = expression(Sea~level~group("(", cm,")")), limits = c(-27.5,7.5)) +
    theme_bw()
arrange(a1, a2, a3, b1, b2, b3, nrow = 2)
#dev.off()

##############################################################################################
# Fit the models using the procedure employed by Vermeer and Rahmstorf
# (adapted from http://www.pnas.org/content/early/2009/12/04/0907765106/suppl/DCSupplemental
# dH/dt = -aTo + aT,    Rahmstorf 2007
# dH/dt = -aTo + a T + b dT/dt = -aTo + a (T + lambda dT/dt),   Vermeer and Rahmstorf 2010

# Rahmstorf 2007 model fit is simpler (remember that observed data was previously
# smoothed and binned)
mR2007 <- lm(dH ~ T, data = subset(data, type == "Binned"))
summary(mR2007)

# Get the correlation between model hindcast and sea level data between 1880 and 2000
# Remember that the intercept of this linear regression is -aTo, the slope is a
# (mimic VR script)
Hhind <- cumsum(predict(mR2007, newdata =
                        subset(data, type == "Smoothed" & t >= 1880 & t <= 2000)))
Ho <- Hhind[111]
Hhind <- Hhind - Ho # normalized to H[1990]
Hobs <- subset(data, type == "Observed" & t >= 1880 & t <= 2000)$H
# Keep parameters and some summaries for later
dftableS1 <- data.frame(Ho = rep(NA, 4), maTo = NA, a = NA, To = NA,
                        b = NA, r2 = NA, rss = NA)
dftableS1[1, ] <- c(Ho, coef(mR2007)[1], coef(mR2007)[2],
                    -coef(mR2007)[1] / coef(mR2007)[2],
                    NA, summary(mR2007)$adj.r.squared, sum((Hhind - Hobs)*(Hhind - Hobs)))
# Ho is the  constant of integration (e.g. intercept)
# maTo is -aTo (mean trend in time)
# a is the coeffcient relating dH and T or H and cumulated temperature anomalies
# To is the equilibrium temperature
# b is the parameter relating dH and dT or H and T
# r2 is the adjusted coefficient of determination from model fit
# rss is the residual sum squares for hindcast

# Now we fit Vermeer and Rahmstorf (2009) model
# lambda: coefficient for linear combination of T and dT/dt (MV)
# dH/dt = -aTo + a T + b dT/dt = -aTo + a (T + lambda dT/dt), so lambda = b/a

# loop for optimal lambda:
rold <- 0
r <- with(subset(data, type == "Binned"), cor(dT, dH))
lambda <- 0
while (r > rold)
{
    rold <- r
    lambda <- lambda - 0.01
    # construct a linear combination of T and dT/dt (MV):
    mtemp <- with(subset(data, type == "Binned"), T + lambda * dT)

    # correlation of these binned data:
    r <- with(subset(data, type == "Binned"), cor(mtemp, dH))

    #A# We have ommited the options employed for model data and
    #A# to 'detrend' in the original code
}
# correlation coefficient (r = 0.9920)
cat("\n\tPearson correlation coefficient r: ", r, "\n");

# (using mtemp above, the linear combination T + lambda * dT)
mVR2009 <- lm(dH ~ mtemp, data = subset(data, type == "Binned"))
summary(mVR2009)
# Get the correlation between model hindcast and sea level data between 1880 and 2000
# Remember that the intercept of this linear regression is -aTo, the slope is a,
# and lambda = b/a.
# (mimic VR script)
mtemp <- with(subset(data, type == "Smoothed" & t >= 1880 & t <= 2000),
              T + lambda * dT)
Hhind <- cumsum(predict(mVR2009, newdata = data.frame(mtemp = mtemp)))
Ho <- Hhind[111]
Hhind <- Hhind - Ho # normalized to H[1990]
Hobs <- subset(data, type == "Observed" & t >= 1880 & t <= 2000)$H

# Keep parameters and some summaries for later
dftableS1[3, ] <- c(Ho, coef(mVR2009)[1], coef(mVR2009)[2],
                    -coef(mVR2009)[1] / coef(mVR2009)[2],
                    coef(mVR2009)[2] * lambda,
                    summary(mVR2009)$adj.r.squared,
                    sum((Hhind - Hobs)*(Hhind - Hobs)))

###############################################################################################
# Fit using the equation in explicit form as a multiple linear regression:
# H = Ho - aTo t + a cT           R2007
# H = Ho - aTo t + a cT + b T     VR2009
# Note that we have not substracted values at time zero because this only
# affects the value of the intercept, Ho (they used sea level at 1990 to
# adjust the mean of the predictions)
meR2007 <- lm(H ~ 1 + t + cT, data = subset(data, type == "Binned"))
summary(meR2007)
meVR2009 <- lm(H ~ 1 + t + cT + T, data = subset(data, type == "Binned"))
summary(meVR2009)

# Now,
Hhind <- predict(meR2007, newdata =
                 subset(data, type == "Smoothed" & t >= 1880 & t <= 2000))
# Keep parameters and some summaries for later
dftableS1[2, ] <- c(coef(meR2007)[1], coef(meR2007)[2], coef(meR2007)[3],
                    -coef(meR2007)[2] / coef(meR2007)[3], NA,
                    summary(meR2007)$adj.r.squared,
                    sum((Hhind - Hobs)*(Hhind - Hobs)))

# and
Hhind <- predict(meVR2009, newdata =
                 subset(data, type == "Smoothed" & t >= 1880 & t <= 2000))
# Keep parameters and some summaries for later
dftableS1[4, ] <- c(coef(meVR2009)[1], coef(meVR2009)[2], coef(meVR2009)[3],
                    -coef(meVR2009)[2] / coef(meVR2009)[3],
                    coef(meVR2009)[4],
                    summary(meVR2009)$adj.r.squared,
                    sum((Hhind - Hobs)*(Hhind - Hobs)))


###############################################################################################
# Create a table (Table S1) to compare the fit obtained by different methods

x11(8,4)
grid.table(signif(dftableS1, digits = 4),
          cols = expression(italic(H)[0], italic(-a*T)[0], italic(a),
              italic(T)[0], italic(b), italic(R)[adj]^2, italic(RSS[Obs])),
           rows = expression(
               italic(dH/dt==a*group("(", T-T[0], ")"))~~group("[", S1, "]"),
               italic(H[t]==H[0]+a*sum(T[i],0,t)-a*T[0]*group("(",t-t[0],")"))~~group("[", S2, "]"),
               italic(dH/dt==a*group("(", T-T[0], ")")+b*dT/dt)~~group("[", S3, "]"),
               italic(H[t]==H[0]+a*sum(T[i],0,t)-a*T[0]*group("(",t-t[0],")")+b*group("(", T[t] - T[t[0]] , ")"))~~group("[", S4, "]")))


grid.text("Table S1. Parameter estimates based on Vermeer and Rahmstorf methods
(ref. S13) and on a multiple regression of the explicit form of their models",
 gp = gpar(font = 2), y = 0.9)


###############################################################################################
# IPCC AR4 based projections
ipcc <- read.table("ipccAR4_scenarios.txt")
str(ipcc)

# Add columns with model projections
# Implicit forms
ipcc$H.iR2007 <- NA # preallocation
ipcc$H.iVR2009 <- NA
# add an indicator for each simulation
ipcc$simulation <- as.factor(with(ipcc, paste(model, scenario, ccycle, sep = "")))
# add the variable mtemp for VR2009 model
ipcc$mtemp <- ipcc$T + lambda * ipcc$dT
for (i in levels(ipcc$simulation)[ ])  # loopo over the 342 different simulations
{
    ipcc$H.iR2007[ipcc$simulation == i] <-
        cumsum(predict(mR2007, newdata = data.frame(T = ipcc$T[ipcc$simulation == i])))
    ipcc$H.iR2007[ipcc$simulation == i] <-
        ipcc$H.iR2007[ipcc$simulation == i] - ipcc$H.iR2007[ipcc$simulation == i][1]
    ipcc$H.iVR2009[ipcc$simulation == i] <-
        cumsum(predict(mVR2009, newdata = data.frame(mtemp = ipcc$mtemp[ipcc$simulation == i])))
    ipcc$H.iVR2009[ipcc$simulation == i] <-
        ipcc$H.iVR2009[ipcc$simulation == i] - ipcc$H.iVR2009[ipcc$simulation == i][1]
}
# Explicit forms
ipcc$H.eVR2009 <- predict(meVR2009, newdata = ipcc)
ipcc$H.eR2007 <- predict(meR2007, newdata = ipcc)

# Prepare a summary of the data
summipcc <- ddply(ipcc, c("t", "scenario"), summarise,
                  H.iVR2009 = mean(H.iVR2009),
                  H.iVR2009.min = min(H.iVR2009),
                  H.iVR2009.max = max(H.iVR2009),
                  H.iVR2009.P25 = quantile(H.iVR2009, 0.25),
                  H.iVR2009.P75 = quantile(H.iVR2009, 0.75),
                  H.iR2007 = mean(H.iR2007),
                  H.iR2007.min = min(H.iR2007),
                  H.iR2007.max = max(H.iR2007),
                  H.iR2007.P25 = quantile(H.iR2007, 0.25),
                  H.iR2007.P75 = quantile(H.iR2007, 0.75),
                  H.eVR2009 = mean(H.eVR2009),
                  H.eVR2009.min = min(H.eVR2009),
                  H.eVR2009.max = max(H.eVR2009),
                  H.eVR2009.P25 = quantile(H.eVR2009, 0.25),
                  H.eVR2009.P75 = quantile(H.eVR2009, 0.75),
                  H.eR2007 = mean(H.eR2007),
                  H.eR2007.min = min(H.eR2007),
                  H.eR2007.max = max(H.eR2007),
                  H.eR2007.P25 = quantile(H.eR2007, 0.25),
                  H.eR2007.P75 = quantile(H.eR2007, 0.75)
                  )

# Extract predicted ranges for different scenarios (arrange data
# for Table S2)
ipcc2100 <- subset(summipcc, t == 2100)
ipcc2100[ , 3:22] <- round(ipcc2100[ , 3:22], digits = 0)
ipcc2100$modelS1 <- with(ipcc2100, as.character(paste(H.iR2007," (", H.iR2007.P25, ", ", H.iR2007.P75, ")", sep = ""))) # R2007, implicit
ipcc2100$modelS2 <- with(ipcc2100, as.character(paste(H.eR2007," (", H.eR2007.P25, ", ", H.eR2007.P75, ")", sep = ""))) # R2007, explicit
ipcc2100$modelS3 <- with(ipcc2100, as.character(paste(H.iVR2009," (", H.iVR2009.P25, ", ", H.iVR2009.P75, ")", sep = "")))# VR2009, implicit
ipcc2100$modelS4 <- with(ipcc2100, as.character(paste(H.eVR2009," (", H.eVR2009.P25, ", ", H.eVR2009.P75, ")", sep = ""))) # VR2009, explicit
ipcc2100 <- ipcc2100[ , c(2, 23:26)]
x11(7,4)
grid.table(ipcc2100, show.rownames = FALSE)
grid.text("Table S2. Sea level ranges projected for year 2100 based on
temperatures projected for different IPCC AR4 scenarios.",
          gp = gpar(font = 2), y = 0.9)



###############################################################################################
## Table S3

# Extract the contribution of each term
data$contrib.eR2007.cT <- with(data, dftableS1$a[2] * cT)
data$contrib.eR2007.T <- NA
data$contrib.eR2007.total <- predict(meR2007, newdata = data)
data$contrib.eVR2009.cT <- with(data, dftableS1$a[4] * cT)
data$contrib.eVR2009.T <- with(data, dftableS1$b[4] * T)
data$contrib.eVR2009.total <- predict(meVR2009, newdata = data)


ipcc$contrib.eR2007.cT <- dftableS1$a[2] * ipcc$cT
ipcc$contrib.eR2007.T <- NA
ipcc$contrib.eVR2009.cT <- dftableS1$a[4] * ipcc$cT
ipcc$contrib.eVR2009.T <- dftableS1$b[4] * ipcc$T
summipcc2 <- ddply(ipcc, c("t"), summarise,
                   H.eVR2009 = mean(H.eVR2009),
                   H.eVR2009.P25 = quantile(H.eVR2009, 0.25),
                   H.eVR2009.P75 = quantile(H.eVR2009, 0.75),
                   H.eVR2009.contrib.cT = mean(contrib.eVR2009.cT),
                   H.eVR2009.contrib.cT.P25 = quantile(contrib.eVR2009.cT, 0.25),
                   H.eVR2009.contrib.cT.P75 = quantile(contrib.eVR2009.cT, 0.75),
                   H.eVR2009.contrib.T = mean(contrib.eVR2009.T),
                   H.eVR2009.contrib.T.P25 = quantile(contrib.eVR2009.T, 0.25),
                   H.eVR2009.contrib.T.P75 = quantile(contrib.eVR2009.T, 0.75),
                   H.eR2007 = mean(H.eR2007),
                   H.eR2007.P25 = quantile(H.eR2007, 0.25),
                   H.eR2007.P75 = quantile(H.eR2007, 0.75),
                   H.eR2007.contrib.cT = mean(contrib.eR2007.cT),
                   H.eR2007.contrib.cT.P25 = quantile(contrib.eR2007.cT, 0.25),
                   H.eR2007.contrib.cT.P75 = quantile(contrib.eR2007.cT, 0.75))

# get increases between 1880 and 2000 and between 2000 and 2100
contrib.1880.2000 <-
    t(signif(diff(as.matrix(subset(data, (t == 1880.5 | t == 2000.5) &
                                   type == "Smoothed")[ , 8:13])),
             digits = 4))

contrib.2000.2100 <-
    t(signif(diff(as.matrix(subset(summipcc2, t == 2100 | t == 2000))),
             digits = 4))

dftableS3 <- data.frame(model = c(rep("Model [S2]", 2), rep("Model [S4]", 2)),
                        period = rep(c("1880-2000", "2000-2100"), 2),
                        cT = c(contrib.1880.2000[1],
                               paste(contrib.2000.2100[14], "\n(", contrib.2000.2100[15], ", ", contrib.2000.2100[16], ")", sep =""),
                               contrib.1880.2000[4],
                               paste(contrib.2000.2100[5], "\n(", contrib.2000.2100[6], ", ", contrib.2000.2100[7], ")", sep ="")),
                        t = signif(c(dftableS1$maTo[2] * c(120, 100), dftableS1$maTo[4] * c(120, 100)), digits = 4),
                        T = c("---",
                              "---",
                               contrib.1880.2000[5],
                              paste(contrib.2000.2100[8], "\n(", contrib.2000.2100[9], ", ", contrib.2000.2100[10], ")", sep ="")),
                        total = c(contrib.1880.2000[3],
                               paste(contrib.2000.2100[11], "\n(", contrib.2000.2100[12], ", ", contrib.2000.2100[13], ")", sep =""),
                               contrib.1880.2000[6],
                               paste(contrib.2000.2100[2], "\n(", contrib.2000.2100[3], ", ", contrib.2000.2100[4], ")", sep ="")))

x11(8,5)
grid.table(dftableS3, show.rownames = FALSE, equal.height = TRUE,
           cols = expression(Model,Period,italic(a*sum(T[i], 0, t)),
               italic(-a*T[0]*group("(", t - t[0], ")")),
               italic(b*group("(", T[t] - T[0], ")")),
               "Sea level increase, cm"))
grid.text("Table S3. Contribution of each term included in the models proposed by Vermeer and
Rahmstorf to explain observed sea level rise (1880-2000) and to projections of future
sea level rise based on IPCC AR4 temperature projections (2000-2100).",
          gp = gpar(font = 2), y = 0.9)

###############################################################################################
## Table S4
# Correlation between variables
# H t dH dT T cT
#signif(cor(na.omit(subset(data, type == "Observed")[, c(2, 1, 3, 5, 4, 6)])), digits = 2)
corObs <- round(cor(na.omit(subset(data, type == "Observed")[,  c(2, 1, 3, 5, 4, 6)])), digits = 3)
corSmo <- round(cor(na.omit(subset(data, type == "Smoothed")[,  c(2, 1, 3, 5, 4, 6)])), digits = 3)
corBin <- round(cor(na.omit(subset(data, type == "Binned")[,  c(2, 1, 3, 5, 4, 6)])), digits = 3)
corMatrix <- matrix(paste(corObs, ", ", corSmo, ", ", corBin), byrow = TRUE, nrow = 6)
diag(corMatrix) <- rep("---", 6)
corMatrix[lower.tri(corMatrix)] <- ""

ipcc$simulation <- as.factor(with(ipcc, paste(model, scenario, ccycle, sep = "")))
summipcc2 <- ddply(ipcc, c("simulation"), summarise,
                   cor.t.dT = cor(t, dT),
                   cor.t.T = cor(t, T),
                   cor.t.cT = cor(t, cT),
                   cor.dT.T = cor(dT, T),
                   cor.dT.cT = cor(dT, cT),
                   cor.T.cT = cor(T, cT))
summary(summipcc2)
pastecors <- function(cordata)
{
    paste(round(mean(cordata), digits = 3), "\n(",
          round(quantile(cordata, 0.25), digits = 3), ", ",
          round(quantile(cordata, 0.75), digits = 3), ")", sep = "")
}
corMatrix[4, 2] <- # t.dT
    pastecors(summipcc2$cor.t.dT)
corMatrix[5, 2] <- # t.T
    pastecors(summipcc2$cor.t.T)
corMatrix[6, 2] <- # t.cT
    pastecors(summipcc2$cor.t.cT)
corMatrix[5, 4] <- # dT.T
    pastecors(summipcc2$cor.dT.T)
corMatrix[6, 4] <- # dT.cT
    pastecors(summipcc2$cor.dT.cT)
corMatrix[6, 5] <- # T.cT
    pastecors(summipcc2$cor.T.cT)

x11(10, 6)
e <- expression(italic(H), italic(t), italic(dH/dt), italic(dT/dt), italic(T), italic(sum(T[i], 0, t)))
grid.table(as.data.frame(corMatrix), cols = e, rows = e, equal.height = TRUE)
grid.text("Table S4. Correlation between the terms included in the different models
during the training (upper diagonal) and the projection (lower diagonal) period.",
          gp = gpar(font = 2), y = 0.9)


###############################################################################################
## Figures S1-S4
preData <- function(data)
{
    # based on code by Mike Laurence;
    # http://ggplot2.googlegroups.com/attach/6bf632a9718dddd6/ggcorplot.R?part=2
    # prepares a data.frame to be plotted as a scatter plot matrix

    #define a helper function (borrowed from the "ez" package)
    ezLev = function(x, new_order)
    {
	for(i in rev(new_order))
        {
            x = relevel(x, ref = i)
	}
	return(x)
    }

    # normalize data
    for(i in 1:length(data))
    {
        data[,i]=(data[ , i] - mean(data[ , i], na.rm = TRUE)) / sd(data[ , i], na.rm = TRUE)
    }
    # obtain new data frame
    z = data.frame()
    i = 1
    j = i
    while(i <= length(data))
    {
        if(j > length(data))
        {
            i=i+1
            j=i
        }
        else
        {
            x = data[,i]
            y = data[,j]
            temp=as.data.frame(cbind(x,y))
            temp=cbind(temp,names(data)[i],names(data)[j])
            z=rbind(z,temp)
            j=j+1
	}
    }
    names(z)=c('x','y','x_lab','y_lab')
    z$x_lab = ezLev(factor(z$x_lab),names(data))
    z$y_lab = ezLev(factor(z$y_lab),names(data))
    z=z[z$x_lab!=z$y_lab,]
    z
}

# subset original data.frame and scale variables
ndata <- data[, c(2, 1, 3, 5, 4, 6, 7)] # H, t, dH, dT, T, cT type

# change variable names to get nice labels
names(ndata) <- c("italic(H)", "italic(t)", "italic(dH/dt)", "italic(dT/dt)",
                  "italic(T)", "italic(sum(T[i], 0, t))", "type")

# use the function above to standardize and prepare the data to build scatterplot matrices
ndataObs <- preData(subset(ndata, type == "Observed")[ , 1:6])
ndataSmo <- preData(subset(ndata, type == "Smoothed")[ , 1:6])
ndataBin <- preData(subset(ndata, type == "Binned")[ , 1:6])

###############################################################################################
# Fig. S1. Scatter plot matrix of the observed, raw data.
# All the variables were standardized to mean zero and standard
# deviation one. The panels confront all variables among each other
# (labels for x and y axes on the top and lateral strips). See Table
# S1 for data sources.
x11(11, 11)
#png("figS1.png", width = 11, height = 11, units = "in", res = 600)
qplot(x = x, y = y, data = ndataObs, colour = I("cadetblue")) +
    facet_grid(x_lab ~ y_lab, scales = "free", labeller = label_parsed) +
    scale_x_continuous("", breaks = seq(-5, 5, by = 1)) +
    scale_y_continuous("", breaks = seq(-5, 5, by = 1)) +
    theme_bw() + opts(strip.background = theme_rect(fill = "cadetblue"),
                      strip.text.x = theme_text(colour = "white", face = "bold", size = 14),
                      strip.text.y = theme_text(colour = "white", face = "bold", size = 14))
#dev.off()

###############################################################################################
# Fig. S2. Scatter plot matrix of the smoothed data. The figure is
# equivalent to Fig. S1, although the data has been smoothed using
# Singular Spectrum Analysis (embedding dimension, 15 years) using
# the procedures described in ref S9. All the variables were standardized
# to mean zero and standard deviation one. The panels confront all
# variables among each other (labels for x and y axes on the top
# and lateral strips). See Table S1 for data sources.
x11(11, 11)
#png("figS2.png", width = 11, height = 11, units = "in", res = 600)
qplot(x = x, y = y, data = ndataSmo, colour = I("cadetblue")) +
    facet_grid(x_lab ~ y_lab, scales = "free", labeller = label_parsed) +
    scale_x_continuous("", breaks = seq(-5, 5, by = 1)) +
    scale_y_continuous("", breaks = seq(-5, 5, by = 1)) +
    theme_bw() + opts(strip.background = theme_rect(fill = "cadetblue"),
                      strip.text.x = theme_text(colour = "white", face = "bold", size = 14),
                      strip.text.y = theme_text(colour = "white", face = "bold", size = 14))
#dev.off()

###############################################################################################
## Fig. S3. Scatter plot matrix of the smoothed and binned data.
#The figure is equivalent to Figs. S1 and S2, although the data
# has been smoothed using Singular Spectrum Analysis (embedding
# dimension, 15 years) using the procedures described in ref S9
# and binned in 15 year intervals. All the variables were standardized
# to mean zero and standard deviation one. The panels confront all
# variables among each other (labels for x and y axes on the top
# and lateral strips). See Table S1 for data sources.
x11(11, 11)
#png("figS3.png", width = 11, height = 11, units = "in", res = 600)
qplot(x = x, y = y, data = ndataBin, colour = I("cadetblue")) +
    facet_grid(x_lab ~ y_lab, scales = "free", labeller = label_parsed) +
    scale_x_continuous("", breaks = seq(-5, 5, by = 1)) +
    scale_y_continuous("", breaks = seq(-5, 5, by = 1)) +
    theme_bw() + opts(strip.background = theme_rect(fill = "cadetblue"),
                      strip.text.x = theme_text(colour = "white", face = "bold", size = 14),
                      strip.text.y = theme_text(colour = "white", face = "bold", size = 14))
#dev.off()

###############################################################################################
## Fig. S4.

# subset original data.frame and scale variables
nipcc <- ipcc[, c(1, 6, 5, 7)] # t, dT, T, cT

# change variable names to get nice labels
names(nipcc) <- c("italic(t)", "italic(dT/dt)", "italic(T)", "italic(sum(T[i], 0, t))")

# use the function above to standardize and prepare the data to build scatterplot matrices
nipcc <- preData(nipcc)

## Fig. S4. Scatter plot matrix of the data employed to project
# future sea level rise, that is, IPCC AR4 model simulations.
# All the variables were standardized to mean zero and standard
# deviation one. To improve the visualization of the distribution
# of the data, alpha blending was applied to tune the transparency
# of each point. In this way, regions with solid color correspond
# to the overlap of at least 100 points. The panels confront all
# variables among each other (labels for x and y axes on the top
# and lateral strips). See Table S2 for data sources.
x11(11, 11)
#png("figS4.png", width = 11, height = 11, units = "in", res = 600)
qplot(x = x, y = y, data = nipcc, colour = I("cadetblue"), alpha = I(1/100)) +
    facet_grid(x_lab ~ y_lab, scales = "free", labeller = label_parsed) +
    scale_x_continuous("", breaks = seq(-5, 5, by = 2.5)) +
    scale_y_continuous("", breaks = seq(-5, 5, by = 2.5)) +
    theme_bw() + opts(strip.background = theme_rect(fill = "cadetblue"),
                      strip.text.x = theme_text(colour = "white", face = "bold", size = 14),
                      strip.text.y = theme_text(colour = "white", face = "bold", size = 14))
#dev.off()


####################################################################################
### Other figures
summipcc <- ddply(ipcc, c("t", "scenario"), summarise,
                  T = mean(T),
                  Tmin = min(T),
                  Tmax = max(T),
                  T.P25 = quantile(T, 0.25),
                  T.P75 = quantile(T, 0.75),
                  H.iVR2009 = mean(H.iVR2009),
                  H.iVR2009.min = min(H.iVR2009),
                  H.iVR2009.max = max(H.iVR2009),
                  H.iVR2009.P25 = quantile(H.iVR2009, 0.25),
                  H.iVR2009.P75 = quantile(H.iVR2009, 0.75),
                  H.iR2007 = mean(H.iR2007),
                  H.iR2007.min = min(H.iR2007),
                  H.iR2007.max = max(H.iR2007),
                  H.iR2007.P25 = quantile(H.iR2007, 0.25),
                  H.iR2007.P75 = quantile(H.iR2007, 0.75),
                  H.eVR2009 = mean(H.eVR2009),
                  H.eVR2009.min = min(H.eVR2009),
                  H.eVR2009.max = max(H.eVR2009),
                  H.eVR2009.P25 = quantile(H.eVR2009, 0.25),
                  H.eVR2009.P75 = quantile(H.eVR2009, 0.75),
                  H.eR2007 = mean(H.eR2007),
                  H.eR2007.min = min(H.eR2007),
                  H.eR2007.max = max(H.eR2007),
                  H.eR2007.P25 = quantile(H.eR2007, 0.25),
                  H.eR2007.P75 = quantile(H.eR2007, 0.75)
                  )


# Plot temperatures
x11(11, 7)
ggplot(data = summipcc, aes(x = t, y = T)) +
    geom_ribbon(aes(ymin = Tmin, ymax = Tmax, fill = scenario), alpha = 0.25) +
    geom_ribbon(aes(ymin = T.P25, ymax = T.P75, fill = scenario), alpha = 0.5) +
    geom_line(aes(colour = scenario), size = 1) +
    facet_wrap(~ scenario, nrow = 2) +
    scale_colour_brewer(name = "SRES scenario") + scale_fill_brewer(name = "SRES scenario") +
    scale_x_continuous(name = "", limits = c(2000, 2100)) +
    scale_y_continuous(name = expression(Delta*italic(T)[1951-1980]~group("[",degree*C,"]"))) +
    opts(title = "Temperature projections from IPCC AR4") + theme_bw()

################
# Fig. S5. Sea level projections based on equation [S2] (see Table S2).
# The lines correspond to mean of several predictions; the inner, darker
# envelope is based on the first and third quartile of the distribution
# of simulated sea levels for each year, and the outer, lighter ribbon
# covers the range of the projections.

# Create a data.frame with IPCC AR4 projected ranges of sea level increase
# (Extracted from the Summary for Policy Makers)
# Note that these model based ranges exclude future rapid dynamical changes
# in ice flow
ipccAR4slr <- data.frame(scenario = c("A1B", "A1FI", "A1T", "A2", "B1", "B2"),
                         t = rep(2095, 6),
                         Hmin = 100*c(0.21, 0.26, 0.20, 0.23, 0.18, 0.20),
                         Hmax = 100*c(0.48, 0.59, 0.45, 0.51, 0.38, 0.43),
                         nlabel = rep("IPCC AR4", 6))
ipccAR4slr$Hmean <- 0.5 * (ipccAR4slr$Hmin + ipccAR4slr$Hmax)

# Plot sea level projections based on Rahmstorf (2007) dual
x11(11, 7)
#png("figS5.png", width = 11, height = 7, units = "in", res = 600)
ggplot(data = summipcc, aes(x = t, y = H.eR2007)) +
    geom_ribbon(aes(ymin = H.eR2007.min, ymax = H.eR2007.max, fill = scenario), alpha = 0.25) +
    geom_ribbon(aes(ymin = H.eR2007.P25, ymax = H.eR2007.P75, fill = scenario), alpha = 0.5) +
    geom_line(aes(colour = scenario), size = 1) +
#    geom_linerange(data = ipccAR4slr, size = I(1.5), aes(x = t, y = NA, ymin = Hmin, ymax = Hmax, colour = scenario)) +
    facet_wrap(~ scenario, nrow = 2) +
    scale_colour_brewer(name = "SRES scenario") + scale_fill_brewer(name = "SRES scenario") +
    scale_x_continuous(name = "", limits = c(2000, 2100)) +
    scale_y_continuous(name = expression(delta*italic(H)[1990]~group("[",cm,"]")), limits = c(0,250)) +
    opts(title = "Sea level projections from Rahmstorf (2007) model and IPCC AR4 temperatures") +
    theme_bw()
#dev.off()

#############
# Fig. S6. Sea level projections based on equation [S4] (see Table S2).
# The lines correspond to mean of several predictions; the inner, darker
# envelope is based on the first and third quartile of the distribution of
# simulated sea levels for each year, and the outer, lighter ribbon covers
# the range of the projections.
# Plot sea level projections based on Vermeer and Rahmstorf (2009) dual model
x11(11, 7)
#png("figS6.png", width = 11, height = 7, units = "in", res = 600)
ggplot(data = summipcc, aes(x = t, y = H.eVR2009)) +
    geom_ribbon(aes(ymin = H.eVR2009.min, ymax = H.eVR2009.max, fill = scenario), alpha = 0.25) +
    geom_ribbon(aes(ymin = H.eVR2009.P25, ymax = H.eVR2009.P75, fill = scenario), alpha = 0.5) +
    geom_line(aes(colour = scenario), size = 1) +
    facet_wrap(~ scenario, nrow = 2) +
    scale_colour_brewer(name = "SRES scenario") + scale_fill_brewer(name = "SRES scenario") +
    scale_x_continuous(name = "", limits = c(2000,2100)) +
    scale_y_continuous(name = expression(delta*italic(H)[1990]~group("[",cm,"]")), limits = c(0,250)) +
    opts(title = "Sea level projections from Vermeer and Rahmstorf (2009) dual model and IPCC AR4 temperatures")+
    theme_bw()
#dev.off()

# Inspect the individual distribution of projections (using alpha blending)
x11(11, 7)
ggplot(data = ipcc, aes(x = t, y = H.eR2007, group = simulation)) +
    geom_line(aes(colour = scenario), alpha = 0.25) +
    facet_wrap(~ scenario, nrow = 2) +
    scale_colour_brewer(name = "SRES scenario") +
    scale_x_continuous(name = "", limits = c(2000,2100)) +
    scale_y_continuous(name = expression(delta*italic(H)[1990]~group("[",cm,"]")), limits = c(0,250)) +
    opts(title = "Sea level projections from Rahmstorf (2007) model and IPCC AR4 temperatures") + theme_bw()

x11(11, 7)
ggplot(data = ipcc, aes(x = t, y = H.eVR2009, group = simulation)) +
    geom_line(aes(colour = scenario), alpha = 0.25) +
    facet_wrap(~ scenario, nrow = 2) +
    scale_colour_brewer(name = "SRES scenario") +
    scale_x_continuous(name = "", limits = c(2000,2100)) +
    scale_y_continuous(name = expression(delta*italic(H)[1990]~group("[",cm,"]")), limits = c(0,250)) +
    opts(title = "Sea level projections from Vermeer and Rahmstorf (2009) dual model and IPCC AR4 temperatures") + theme_bw()

###############################################################################################
